@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2014 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@ static bool isaddr(const Variable *var)
 /** Is given variable an integer variable */
 static bool isint(const Variable *var)
 {
-    return (var && var->isIntegralType() && !var->isArrayOrPointer());
+    return (var && var->isIntegralType() && !var->isArrayOrPointer() && var->typeStartToken()->str() != "bool");
 }
 
 void Check64BitPortability::pointerassignment()
