@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2014 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -283,6 +283,32 @@ void CheckInternal::checkRedundantNextPrevious()
     }
 }
 
+void CheckInternal::checkExtraWhitespace()
+{
+    for (const Token *tok = _tokenizer->tokens(); tok; tok = tok->next()) {
+        if (!Token::simpleMatch(tok, "Token :: simpleMatch (") &&
+            !Token::simpleMatch(tok, "Token :: findsimplematch (") &&
+            !Token::simpleMatch(tok, "Token :: Match (") &&
+            !Token::simpleMatch(tok, "Token :: findmatch ("))
+            continue;
+
+        const std::string& funcname = tok->strAt(2);
+
+        // Get pattern string
+        const Token *pattern_tok = tok->tokAt(4)->nextArgument();
+        if (!pattern_tok || pattern_tok->type() != Token::eString)
+            continue;
+
+        const std::string pattern = pattern_tok->strValue();
+        if (!pattern.empty() && (pattern[0] == ' ' || *pattern.rbegin() == ' '))
+            extraWhitespaceError(tok, pattern, funcname);
+
+        // two whitespaces or more
+        if (pattern.find("  ") != std::string::npos)
+            extraWhitespaceError(tok, pattern, funcname);
+    }
+}
+
 void CheckInternal::multiComparePatternError(const Token* tok, const std::string& pattern, const std::string &funcname)
 {
     reportError(tok, Severity::error, "multiComparePatternError",
@@ -329,4 +355,11 @@ void CheckInternal::orInComplexPattern(const Token* tok, const std::string& patt
                 "Token::" + funcname + "() pattern \"" + pattern + "\" contains \"||\" or \"|\". Replace it by \"%oror%\" or \"%or%\".");
 }
 
+void CheckInternal::extraWhitespaceError(const Token* tok, const std::string& pattern, const std::string &funcname)
+{
+    reportError(tok, Severity::warning, "extraWhitespaceError",
+                "Found extra whitespace inside Token::" + funcname + "() call: \"" + pattern + "\""
+               );
+}
+
 #endif // #ifdef CHECK_INTERNAL
