@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2014 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -73,7 +73,7 @@ public:
         checkOther.checkVarFuncNullUB();
         checkOther.checkNanInArithmeticExpression();
         checkOther.checkCommaSeparatedReturn();
-        checkOther.checkReturnIgnoredReturnValue();
+        checkOther.checkIgnoredReturnValue();
     }
 
     /** @brief Run checks against the simplified token list */
@@ -231,7 +231,7 @@ public:
     void checkComparisonFunctionIsAlwaysTrueOrFalse();
 
     /** @brief %Check for ignored return values. */
-    void checkReturnIgnoredReturnValue();
+    void checkIgnoredReturnValue();
 
 private:
     // Error messages..
