@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2014 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -190,10 +190,10 @@ void CheckSizeof::checkSizeofForPointerSize()
             // looks suspicious
             // Do it for first variable
             if (variable && (Token::Match(tokSize, "sizeof ( &| %varid% )", variable->varId()) ||
-                             Token::Match(tokSize, "sizeof &| %varid%", variable->varId()))) {
+                             Token::Match(tokSize, "sizeof &| %varid% !!.", variable->varId()))) {
                 sizeofForPointerError(variable, variable->str());
             } else if (variable2 && (Token::Match(tokSize, "sizeof ( &| %varid% )", variable2->varId()) ||
-                                     Token::Match(tokSize, "sizeof &| %varid%", variable2->varId()))) {
+                                     Token::Match(tokSize, "sizeof &| %varid% !!.", variable2->varId()))) {
                 sizeofForPointerError(variable2, variable2->str());
             }
         }
