@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2014 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 
 #include "preprocessor.h" // Preprocessor
 #include "tokenize.h" // Tokenizer
-#include "checkunusedfunctions.h"
 
 #include "check.h"
 #include "path.h"
@@ -48,6 +47,10 @@ CppCheck::CppCheck(ErrorLogger &errorLogger, bool useGlobalSuppressions)
 
 CppCheck::~CppCheck()
 {
+    while (!fileInfo.empty()) {
+        delete fileInfo.back();
+        fileInfo.pop_back();
+    }
     S_timerResults.ShowResults(_settings._showtime);
 }
 
@@ -274,23 +277,6 @@ void CppCheck::internalError(const std::string &filename, const std::string &msg
     }
 }
 
-
-void CppCheck::checkFunctionUsage()
-{
-    // This generates false positives - especially for libraries
-    if (_settings.isEnabled("unusedFunction") && _settings._jobs == 1) {
-        const bool verbose_orig = _settings._verbose;
-        _settings._verbose = false;
-
-        if (_settings._errorsOnly == false)
-            _errorLogger.reportOut("Checking usage of global functions..");
-
-        CheckUnusedFunctions::instance.check(this);
-
-        _settings._verbose = verbose_orig;
-    }
-}
-
 void CppCheck::analyseFile(std::istream &fin, const std::string &filename)
 {
     // Preprocess file..
@@ -309,19 +295,6 @@ void CppCheck::analyseFile(std::istream &fin, const std::string &filename)
     std::istringstream istr(code);
     tokenizer.tokenize(istr, filename.c_str());
     tokenizer.simplifyTokenList2();
-
-    // Analyse the tokens..
-    std::set<std::string> data;
-    for (std::list<Check *>::const_iterator it = Check::instances().begin(); it != Check::instances().end(); ++it) {
-        (*it)->analyse(tokenizer.tokens(), data);
-    }
-
-    // Save analysis results..
-    // TODO: This loop should be protected by a mutex or something like that
-    //       The saveAnalysisData must _not_ be called from many threads at the same time.
-    for (std::list<Check *>::const_iterator it = Check::instances().begin(); it != Check::instances().end(); ++it) {
-        (*it)->saveAnalysisData(data);
-    }
 }
 
 //---------------------------------------------------------------------------
@@ -388,8 +361,12 @@ bool CppCheck::checkFile(const std::string &code, const char FileName[], std::se
             (*it)->runChecks(&_tokenizer, &_settings, this);
         }
 
-        if (_settings.isEnabled("unusedFunction") && _settings._jobs == 1)
-            CheckUnusedFunctions::instance.parseTokens(_tokenizer, FileName, &_settings);
+        // Analyse the tokens..
+        for (std::list<Check *>::const_iterator it = Check::instances().begin(); it != Check::instances().end(); ++it) {
+            Check::FileInfo *fi = (*it)->getFileInfo(&_tokenizer, &_settings);
+            if (fi != nullptr)
+                fileInfo.push_back(fi);
+        }
 
         executeRules("normal", _tokenizer);
 
@@ -690,3 +667,11 @@ void CppCheck::getErrorMessages()
     Tokenizer::getErrorMessages(this, &_settings);
     Preprocessor::getErrorMessages(this, &_settings);
 }
+
+void CppCheck::analyseWholeProgram()
+{
+    // Analyse the tokens..
+    for (std::list<Check *>::const_iterator it = Check::instances().begin(); it != Check::instances().end(); ++it)
+        (*it)->analyseWholeProgram(fileInfo, *this);
+}
+
