@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2014 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include "tokenlist.h"
 #include "mathlib.h"
 #include "token.h"
+#include "symboldatabase.h"
 
 #include <string>
 #include <algorithm>
@@ -46,6 +47,7 @@ Library::Error Library::load(const char exename[], const char path[])
         return Error();
     }
 
+    std::string absolute_path;
     // open file..
     tinyxml2::XMLDocument doc;
     tinyxml2::XMLError error = doc.LoadFile(path);
@@ -55,6 +57,8 @@ Library::Error Library::load(const char exename[], const char path[])
         if (Path::getFilenameExtension(fullfilename) == "") {
             fullfilename += ".cfg";
             error = doc.LoadFile(fullfilename.c_str());
+            if (error != tinyxml2::XML_ERROR_FILE_NOT_FOUND)
+                absolute_path = Path::getAbsoluteFilePath(fullfilename.c_str());
         }
 
         if (error == tinyxml2::XML_ERROR_FILE_NOT_FOUND) {
@@ -69,12 +73,24 @@ Library::Error Library::load(const char exename[], const char path[])
             const char *sep = (!cfgfolder.empty() && cfgfolder[cfgfolder.size()-1U]=='/' ? "" : "/");
             const std::string filename(cfgfolder + sep + fullfilename);
             error = doc.LoadFile(filename.c_str());
+            if (error != tinyxml2::XML_ERROR_FILE_NOT_FOUND)
+                absolute_path = Path::getAbsoluteFilePath(filename.c_str());
         }
+    } else
+        absolute_path = Path::getAbsoluteFilePath(path);
+
+    if (error == tinyxml2::XML_NO_ERROR) {
+        if (_files.find(absolute_path) == _files.end()) {
+            Error err = load(doc);
+            if (err.errorcode == OK)
+                _files.insert(absolute_path);
+            return err;
+        }
+
+        return Error(OK); // ignore duplicates
     }
 
-    return (error == tinyxml2::XML_NO_ERROR) ?
-           load(doc) :
-           Error(error == tinyxml2::XML_ERROR_FILE_NOT_FOUND ? FILE_NOT_FOUND : BAD_XML);
+    return Error(error == tinyxml2::XML_ERROR_FILE_NOT_FOUND ? FILE_NOT_FOUND : BAD_XML);
 }
 
 bool Library::loadxmldata(const char xmldata[], std::size_t len)
@@ -102,7 +118,7 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
         return Error(UNSUPPORTED_FORMAT);
 
     for (const tinyxml2::XMLElement *node = rootnode->FirstChildElement(); node; node = node->NextSiblingElement()) {
-        std::string nodename = node->Name();
+        const std::string nodename = node->Name();
         if (nodename == "memory" || nodename == "resource") {
             // get allocationId to use..
             int allocationId = 0;
@@ -116,7 +132,7 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
                 }
             }
             if (allocationId == 0) {
-                if (strcmp(node->Name(), "memory")==0)
+                if (nodename == "memory")
                     while (!ismemory(++allocid));
                 else
                     while (!isresource(++allocid));
@@ -125,18 +141,19 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
 
             // add alloc/dealloc/use functions..
             for (const tinyxml2::XMLElement *memorynode = node->FirstChildElement(); memorynode; memorynode = memorynode->NextSiblingElement()) {
-                if (strcmp(memorynode->Name(),"alloc")==0) {
+                const std::string memorynodename = memorynode->Name();
+                if (memorynodename == "alloc") {
                     _alloc[memorynode->GetText()] = allocationId;
                     const char *init = memorynode->Attribute("init");
                     if (init && strcmp(init,"false")==0) {
                         returnuninitdata.insert(memorynode->GetText());
                     }
-                } else if (strcmp(memorynode->Name(),"dealloc")==0)
+                } else if (memorynodename == "dealloc")
                     _dealloc[memorynode->GetText()] = allocationId;
-                else if (strcmp(memorynode->Name(),"use")==0)
+                else if (memorynodename == "use")
                     use.insert(memorynode->GetText());
                 else
-                    return Error(BAD_ELEMENT, memorynode->Name());
+                    return Error(BAD_ELEMENT, memorynodename);
             }
         }
 
@@ -161,7 +178,7 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
             std::string name = name_char;
 
             for (const tinyxml2::XMLElement *functionnode = node->FirstChildElement(); functionnode; functionnode = functionnode->NextSiblingElement()) {
-                std::string functionnodename = functionnode->Name();
+                const std::string functionnodename = functionnode->Name();
                 if (functionnodename == "noreturn")
                     _noreturn[name] = (strcmp(functionnode->GetText(), "true") == 0);
                 else if (functionnodename == "pure")
@@ -184,7 +201,7 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
                     std::string& valid = argumentChecks[name][nr].valid;
                     std::list<ArgumentChecks::MinSize>& minsizes = argumentChecks[name][nr].minsizes;
                     for (const tinyxml2::XMLElement *argnode = functionnode->FirstChildElement(); argnode; argnode = argnode->NextSiblingElement()) {
-                        std::string argnodename = argnode->Name();
+                        const std::string argnodename = argnode->Name();
                         if (argnodename == "not-bool")
                             notbool = true;
                         else if (argnodename == "not-null")
@@ -256,7 +273,7 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
                         }
 
                         else
-                            return Error(BAD_ATTRIBUTE, argnode->Name());
+                            return Error(BAD_ATTRIBUTE, argnodename);
                     }
                     argumentChecks[name][nr].notbool   = notbool;
                     argumentChecks[name][nr].notnull   = notnull;
@@ -270,7 +287,7 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
                     const tinyxml2::XMLAttribute* secure = functionnode->FindAttribute("secure");
                     _formatstr[name] = std::make_pair(scan && scan->BoolValue(), secure && secure->BoolValue());
                 } else
-                    return Error(BAD_ELEMENT, functionnode->Name());
+                    return Error(BAD_ELEMENT, functionnodename);
             }
         }
 
@@ -299,7 +316,7 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
             _processAfterCode[extension] = (aftercode && strcmp(aftercode, "true") == 0);
 
             for (const tinyxml2::XMLElement *markupnode = node->FirstChildElement(); markupnode; markupnode = markupnode->NextSiblingElement()) {
-                std::string markupnodename = markupnode->Name();
+                const std::string markupnodename = markupnode->Name();
                 if (markupnodename == "keywords") {
                     for (const tinyxml2::XMLElement *librarynode = markupnode->FirstChildElement(); librarynode; librarynode = librarynode->NextSiblingElement()) {
                         if (strcmp(librarynode->Name(), "keyword") == 0) {
@@ -322,12 +339,13 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
                             return Error(MISSING_ATTRIBUTE, "prefix");
 
                         for (const tinyxml2::XMLElement *e = exporter->FirstChildElement(); e; e = e->NextSiblingElement()) {
-                            if (strcmp(e->Name(), "prefix") == 0)
+                            const std::string ename = e->Name();
+                            if (ename == "prefix")
                                 _exporters[prefix].addPrefix(e->GetText());
-                            else if (strcmp(e->Name(), "suffix") == 0)
+                            else if (ename == "suffix")
                                 _exporters[prefix].addSuffix(e->GetText());
                             else
-                                return Error(BAD_ELEMENT, e->Name());
+                                return Error(BAD_ELEMENT, ename);
                         }
                     }
                 }
@@ -343,11 +361,12 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
 
                 else if (markupnodename == "codeblocks") {
                     for (const tinyxml2::XMLElement *blocknode = markupnode->FirstChildElement(); blocknode; blocknode = blocknode->NextSiblingElement()) {
-                        if (strcmp(blocknode->Name(), "block") == 0) {
+                        const std::string blocknodename = blocknode->Name();
+                        if (blocknodename == "block") {
                             const char * blockName = blocknode->Attribute("name");
                             if (blockName)
                                 _executableblocks[extension].addBlock(blockName);
-                        } else if (strcmp(blocknode->Name(), "structure") == 0) {
+                        } else if (blocknodename == "structure") {
                             const char * start = blocknode->Attribute("start");
                             if (start)
                                 _executableblocks[extension].setStart(start);
@@ -360,12 +379,12 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
                         }
 
                         else
-                            return Error(BAD_ELEMENT, blocknode->Name());
+                            return Error(BAD_ELEMENT, blocknodename);
                     }
                 }
 
                 else
-                    return Error(BAD_ELEMENT, markupnode->Name());
+                    return Error(BAD_ELEMENT, markupnodename);
             }
         }
 
@@ -383,8 +402,62 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
             podtypes[name] = podType;
         }
 
+        else if (nodename == "platformtype") {
+            const char * const type_name = node->Attribute("name");
+            if (type_name == nullptr)
+                return Error(MISSING_ATTRIBUTE, "name");
+            const char *value = node->Attribute("value");
+            if (value == nullptr)
+                return Error(MISSING_ATTRIBUTE, "value");
+            PlatformType type;
+            type._type = value;
+            std::set<std::string> platform;
+            for (const tinyxml2::XMLElement *typenode = node->FirstChildElement(); typenode; typenode = typenode->NextSiblingElement()) {
+                const std::string typenodename = typenode->Name();
+                if (typenodename == "platform") {
+                    const char * const type_attribute = typenode->Attribute("type");
+                    if (type_attribute == nullptr)
+                        return Error(MISSING_ATTRIBUTE, "type");
+                    platform.insert(type_attribute);
+                } else if (typenodename == "signed")
+                    type._signed = true;
+                else if (typenodename == "unsigned")
+                    type._unsigned = true;
+                else if (typenodename == "long")
+                    type._long = true;
+                else if (typenodename == "pointer")
+                    type._pointer= true;
+                else if (typenodename == "ptr_ptr")
+                    type._ptr_ptr = true;
+                else if (typenodename == "const_ptr")
+                    type._const_ptr = true;
+                else
+                    return Error(BAD_ELEMENT, typenodename);
+            }
+            if (platform.empty()) {
+                const PlatformType * const type_ptr = platform_type(type_name, "");
+                if (type_ptr) {
+                    if (*type_ptr == type)
+                        return Error(DUPLICATE_PLATFORM_TYPE, type_name);
+                    return Error(PLATFORM_TYPE_REDEFINED, type_name);
+                }
+                platform_types[type_name] = type;
+            } else {
+                std::set<std::string>::const_iterator it;
+                for (it = platform.begin(); it != platform.end(); ++it) {
+                    const PlatformType * const type_ptr = platform_type(type_name, *it);
+                    if (type_ptr) {
+                        if (*type_ptr == type)
+                            return Error(DUPLICATE_PLATFORM_TYPE, type_name);
+                        return Error(PLATFORM_TYPE_REDEFINED, type_name);
+                    }
+                    platforms[*it]._platform_types[type_name] = type;
+                }
+            }
+        }
+
         else
-            return Error(BAD_ELEMENT, node->Name());
+            return Error(BAD_ELEMENT, nodename);
     }
     return Error(OK);
 }
@@ -454,7 +527,7 @@ bool Library::isScopeNoReturn(const Token *end, std::string *unknownFunc) const
         if (funcname->str() == "exit")
             return true;
         if (!isnotnoreturn(funcname->str())) {
-            if (unknownFunc && !isnoreturn(funcname->str()))
+            if (unknownFunc && !(isnoreturn(funcname->str()) || (funcname->function() && funcname->function()->isAttributeNoreturn())))
                 *unknownFunc = funcname->str();
             return true;
         }
