@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2014 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,6 +61,7 @@ private:
         TEST_CASE(test5);
         TEST_CASE(test6);
         TEST_CASE(test_numeric);
+        TEST_CASE(void0); // #6327: No fp for statement "(void)0;"
         TEST_CASE(intarray);
         TEST_CASE(structarraynull);
         TEST_CASE(structarray);
@@ -150,6 +151,14 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void void0() { // #6327
+        check("void f() { (void*)0; }");
+        ASSERT_EQUALS("", errout.str());
+
+        check("void f() { $0; }");
+        ASSERT_EQUALS("", errout.str());
+    }
+
     void intarray() {
         check("int arr[] = { 100/2, 1*100 };");
         ASSERT_EQUALS("", errout.str());
@@ -185,6 +194,14 @@ private:
               "}");
         ASSERT_EQUALS("", errout.str());
 
+        // #6260 - C++11 array initialization
+        check("void foo() {\n"
+              "    static const char* a[][2] {\n"
+              "        {\"b\", \"\"},\n"
+              "    };\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
         // #2482 - false positive for empty struct
         check("struct A {};");
         ASSERT_EQUALS("", errout.str());
