@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2014 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@ private:
         TEST_CASE(sizeofsizeof);
         TEST_CASE(sizeofCalculation);
         TEST_CASE(checkPointerSizeof);
+        TEST_CASE(checkPointerSizeofStruct);
         TEST_CASE(sizeofDivisionMemset);
         TEST_CASE(sizeofForArrayParameter);
         TEST_CASE(sizeofForNumericParameter);
@@ -502,6 +503,30 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void checkPointerSizeofStruct() {
+        check("void f() {\n"
+              "    struct foo *ptr;\n"
+              "    memset( ptr->bar, 0, sizeof ptr->bar );\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("void f() {\n"
+              "    struct foo {\n"
+              "        char bar[10];\n"
+              "    };\n"
+              "    memset( ptr->bar, 0, sizeof ptr->bar );\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("void f() {\n"
+              "    struct foo {\n"
+              "        char *bar;\n"
+              "    };\n"
+              "    memset( ptr->bar, 0, sizeof ptr->bar );\n"
+              "}");
+        TODO_ASSERT_EQUALS("[test.cpp:5]: (warning) Size of pointer 'bar' used instead of size of its data.\n", "", errout.str());
+    }
+
     void sizeofDivisionMemset() {
         check("void foo(memoryMapEntry_t* entry, memoryMapEntry_t* memoryMapEnd) {\n"
               "    memmove(entry, entry + 1, (memoryMapEnd - entry) / sizeof(entry));\n"
