@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2014 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -121,34 +121,51 @@ private:
     }
 
     void functionpointer() {
+        check("void foo() { }\n"
+              "int main() {\n"
+              "    f(&foo);\n"
+              "    return 0\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("void foo() { }\n"
+              "int main() {\n"
+              "    f(&::foo);\n"
+              "    return 0\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
         check("namespace abc {\n"
-              "void foo() { }\n"
+              "    void foo() { }\n"
               "};\n"
-              "\n"
-              "int main()\n"
-              "{\n"
+              "int main() {\n"
               "    f(&abc::foo);\n"
               "    return 0\n"
               "}");
         ASSERT_EQUALS("", errout.str());
 
         check("namespace abc {\n"
-              "void foo() { }\n"
+              "    void foo() { }\n"
               "};\n"
-              "\n"
-              "int main()\n"
-              "{\n"
+              "int main() {\n"
               "    f = &abc::foo;\n"
               "    return 0\n"
               "}");
         ASSERT_EQUALS("", errout.str());
 
-        check("namespace abc {\n"  // #3875
-              "void foo() { }\n"
+        check("namespace abc {\n"
+              "    void foo() { }\n"
               "};\n"
-              "\n"
-              "int main()\n"
-              "{\n"
+              "int main() {\n"
+              "    f = &::abc::foo;\n"
+              "    return 0\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("namespace abc {\n"  // #3875
+              "    void foo() { }\n"
+              "};\n"
+              "int main() {\n"
               "    f(abc::foo);\n"
               "    return 0\n"
               "}");
